@@ -1,22 +1,20 @@
 #!/usr/bin/env perl
-# $Id: tlmgrgui.pl 34070 2014-05-16 23:49:53Z karl $
+# $Id: tlmgrgui.pl 41232 2016-05-18 06:04:47Z preining $
 #
-# Copyright 2009-2014 Norbert Preining
+# Copyright 2009-2016 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 #
 # GUI for tlmgr
 # version 2, completely rewritten GUI
 #
-# TODO: Fix return codes of execute_action sub items and execute_action
-#       so that we can give a decent warning window
 # TODO: implement path adjustment also for Windows
 
 $^W = 1;
 use strict;
 
-my $guisvnrev = '$Revision: 34070 $';
-my $guidatrev = '$Date: 2014-05-17 01:49:53 +0200 (Sat, 17 May 2014) $';
+my $guisvnrev = '$Revision: 41232 $';
+my $guidatrev = '$Date: 2016-05-18 08:04:47 +0200 (Wed, 18 May 2016) $';
 my $tlmgrguirevision;
 if ($guisvnrev =~ m/: ([0-9]+) /) {
   $tlmgrguirevision = $1;
@@ -88,7 +86,6 @@ my @htype = qw/-relief ridge/;
 # the list of packages as shown by TixGrid
 #
 my %Packages;
-my $taxonomy;
 my $mw;
 my $tlmgrrev;
 my $menu;
@@ -126,7 +123,6 @@ my $show_collections = 1;
 my $show_schemes = 1;
 my $match_descriptions = 1;
 my $match_filenames = 1;
-my $match_taxonomies = 1;
 my $match_text = "";
 my $selection_value = 0;
 
@@ -135,6 +131,7 @@ my $selection_value = 0;
 $::opt_lang = $config{"gui-lang"} if (defined($config{"gui-lang"}));
 $::opt_lang = $opts{"gui-lang"} if (defined($opts{"gui-lang"}));
 require("TeXLive/trans.pl");
+load_translations();
 
 
 my @archsavail;
@@ -209,7 +206,6 @@ sub guimain {
     $::we_can_save = 1;
   } else {
     $::we_can_save = 0;
-    # here we should pop up a warning window!!!
   }
   $::action_button_state = ($::we_can_save ? "normal" : "disabled");
 
@@ -217,10 +213,6 @@ sub guimain {
   chomp($tlmgrrev);
 
   setup_menu_system();
-  $taxonomy = load_taxonomy_datafile();
-  if (!defined($taxonomy)) {
-    info(__("Cannot load taxonomy file") . "\n");
-  }
   do_rest_of_gui();
   $bgcolor = $loaded_text->cget('-background');
 
@@ -239,7 +231,7 @@ sub guimain {
 
   if (!$::we_can_save) {
     my $no_write_warn = $mw->Dialog(-title => __("Warning"),
-      -text => __("You don't have permissions to change the installation in any way,\nspecifically, the directory %s is not writable.\nPlease run this program as administrator, or contact your local admin.\n\nMost buttons will be disabled.", "$Master/tlpkg/"),
+      -text => __("You don't have permissions to change the installation in any way;\nspecifically, the directory %s is not writable.\nPlease run this program as administrator, or contact your local admin.\n\nMost buttons will be disabled.", "$Master/tlpkg/"),
       -buttons => [ __("Ok") ])->Show();
   }
 
@@ -250,6 +242,20 @@ sub guimain {
 ############## GUI ########################
 
 sub build_initial_gui {
+  # processed @::SAVEDARGV to replace 
+  #  --font='foobar'
+  # with 
+  #  --font 'foobar'
+  # as required by Tk::CmdLine.
+  my @a;
+  for my $c (@::SAVEDARGV) {
+    if ($c =~ m/^--?(font|background|class|display|screen|foreground|geometry|name|title|xrm)=(.*)$/) {
+      push @a, "--$1", $2;
+    } else {
+      push @a, $c;
+    }
+  }
+  Tk::CmdLine::SetArguments(@a);
   $mw = MainWindow->new;
   $mw->title("TeX Live Manager $TeXLive::TLConfig::ReleaseYear");
   $mw->withdraw;
@@ -341,10 +347,6 @@ sub do_rest_of_gui {
   $filter_match->Checkbutton(-text => __("descriptions"),
           -command => \&update_grid,
           -variable => \$match_descriptions)->pack(@a_w);
-  $filter_match->Checkbutton(-text => __("taxonomies"),
-          -command => \&update_grid,
-          -state => (defined($taxonomy) ? "normal" : "disabled"),
-          -variable => \$match_taxonomies)->pack(@a_w);
   $filter_match->Checkbutton(-text => __("filenames"),
           -command => \&update_grid,
           -variable => \$match_filenames)->pack(@a_w);
@@ -354,18 +356,20 @@ sub do_rest_of_gui {
       my ($new_val, undef, $old_val) = @_;
     #   if (!$new_val) {
     #     $match_descriptions = 0;
-    #     $match_taxonomies = 0;
     #     $match_filenames = 0;
     #   } else {
     #    # if something is already in the search field don't change selection
     #     if (!$old_val) {
     #       $match_descriptions = 1;
-    #       $match_taxonomies = 1 if (defined($taxonomy));
     #       $match_filenames = 1;
     #     }
     #   }
-      $match_text = $new_val;
-      update_grid(); return 1; });
+      my $new_match_text = ( length($new_val) >= 3 ? $new_val : "" );
+      if ($new_match_text ne $match_text) {
+        $match_text = $new_match_text;
+        update_grid(); 
+      }
+      return 1; });
 
   my $filter_selection = $filter_frame->Labelframe(-text => __("Selection"));
   if ($mode_expert) { $filter_selection->pack(@left, @x_y, @p_ii); }
@@ -394,7 +398,6 @@ sub do_rest_of_gui {
                       $show_schemes = 1;
                       $selection_value = 0;
                       $match_descriptions = 1;
-                      $match_taxonomies = 1;
                       $match_filenames = 1;
                       update_grid();
                     })->pack(@x_x, @a_c);
@@ -689,26 +692,26 @@ sub show_extended_info {
   #my $t = $sw->Scrolled('ROText', -scrollbars => "oe", -height => 10,
   #  -width => 50, -wrap => 'word', -relief => 'flat');
   #$t->insert("1.0", $tlp->longdesc);
-  #$sw->Label(-text => "Long Desc:")->grid($t, -sticky => 'nw');
+  #$sw->Label(-text => "Long desc:")->grid($t, -sticky => 'nw');
   $tf->Label(-text => __("Long description:"))->grid(
     $tf->Label(-wraplength => '500', -justify => 'left', 
       -text => $tlp->longdesc), -sticky => "nw");
   $tf->Label(-text => __("Installed:"))->grid(
     $tf->Label(-text => ($Packages{$p}{'installed'} ? __("Yes") : __("No"))),
     -sticky => "nw");
-  $tf->Label(-text => __("Local Revision:"))->grid(
+  $tf->Label(-text => __("Local revision:"))->grid(
     $tf->Label(-text => $Packages{$p}{'localrevision'}),
     -sticky => "nw");
   if (defined($Packages{$p}{'localcatalogueversion'})) {
-    $tf->Label(-text => __("Local Catalogue Version:"))->grid(
+    $tf->Label(-text => __("Local Catalogue version:"))->grid(
       $tf->Label(-text => $Packages{$p}{'localcatalogueversion'}),
       -sticky => "nw");
   }
-  $tf->Label(-text => __("Remote Revision:"))->grid(
+  $tf->Label(-text => __("Remote revision:"))->grid(
     $tf->Label(-text => $Packages{$p}{'remoterevisionstring'}),
     -sticky => "nw");
   if (defined($Packages{$p}{'remotecatalogueversion'})) {
-    $tf->Label(-text => __("Remote Catalogue Version:"))->grid(
+    $tf->Label(-text => __("Remote Catalogue version:"))->grid(
       $tf->Label(-text => $Packages{$p}{'remotecatalogueversion'}),
       -sticky => "nw");
   }
@@ -897,18 +900,19 @@ sub update_grid {
     }
   }
 }
+
+sub maybe_strip_last_plus {
+  my $v = shift;
+  if ($v =~ m/\+$/) {
+    chop($v);
+    # just for comparison add one to the version of there is a "+"
+    $v++;
+  }
+  return $v;
+}
   
 sub MatchesFilters {
   my $p = shift;
-  sub maybe_strip_last_plus {
-    my $v = shift;
-    if ($v =~ m/\+$/) {
-      chop($v);
-      # just for comparison add one to the version of there is a "+"
-      $v++;
-    }
-    return $v;
-  }
   # we have to take care since strings in revision numbers on the remote
   # and might contain "+" indicating sub-package updates
   # status
@@ -947,7 +951,7 @@ sub MatchesFilters {
   #   + no search target selected
   #     -> show empty list (maybe show warning "select something")
   #
-  if ($match_descriptions || $match_taxonomies || $match_filenames) {
+  if ($match_descriptions || $match_filenames) {
     my $found = 0;
     my $r = $match_text;
     if ($r eq "") {
@@ -963,14 +967,7 @@ sub MatchesFilters {
         $found = 1;
       }
     }
-    if (!$found) {
-      if (defined($taxonomy) && $match_taxonomies) {
-        if ($Packages{$p}{'taxonomy'} =~ m/$r/i) {
-          $found = 1;
-        }
-      }
-    }
-    # if we already dound, don't check the next condition!
+    # if we already found something, don't check the next condition!
     if (!$found) {
       if ($match_filenames) {
         if ($Packages{$p}{'match_files'} =~ m/$r/i) {
@@ -1299,31 +1296,32 @@ sub apply_settings_changes {
 
 sub init_paper_xdvi {
   if (!win32()) {
-    @{$papers{"xdvi"}} = TeXLive::TLPaper::get_paper_list("xdvi");
-    $currentpaper{"xdvi"} = ${$papers{"xdvi"}}[0];
+    $papers{"xdvi"} = TeXLive::TLPaper::get_paper_list("xdvi");
+    $currentpaper{"xdvi"} = $papers{"xdvi"}->[0];
   }
 }
 sub init_paper_pdftex {
-  @{$papers{"pdftex"}} = TeXLive::TLPaper::get_paper_list("pdftex");
-  $currentpaper{"pdftex"} = ${$papers{"pdftex"}}[0];
+  $papers{"pdftex"} = TeXLive::TLPaper::get_paper_list("pdftex");
+  $currentpaper{"pdftex"} = $papers{"pdftex"}->[0];
 }
 sub init_paper_dvips {
-  @{$papers{"dvips"}} = TeXLive::TLPaper::get_paper_list("dvips");
-  $currentpaper{"dvips"} = ${$papers{"dvips"}}[0];
+  $papers{"dvips"} = TeXLive::TLPaper::get_paper_list("dvips");
+  $currentpaper{"dvips"} = $papers{"dvips"}->[0];
 }
 sub init_paper_context {
   if (defined($localtlpdb->get_package("bin-context"))) {
-    @{$papers{"context"}} = TeXLive::TLPaper::get_paper_list("context");
-    $currentpaper{"context"} = ${$papers{"context"}}[0];
+    $papers{"context"} = TeXLive::TLPaper::get_paper_list("context");
+    $currentpaper{"context"} = $papers{"context"}->[0];
   }
 }
 sub init_paper_dvipdfmx {
-  @{$papers{"dvipdfmx"}} = TeXLive::TLPaper::get_paper_list("dvipdfmx");
-  $currentpaper{"dvipdfmx"} = ${$papers{"dvipdfmx"}}[0];
+  $papers{"dvipdfmx"} = TeXLive::TLPaper::get_paper_list("dvipdfmx");
+  $currentpaper{"dvipdfmx"} = $papers{"dvipdfmx"}->[0];
 }
+
 sub init_paper_psutils {
-  @{$papers{"psutils"}} = TeXLive::TLPaper::get_paper_list("psutils");
-  $currentpaper{"psutils"} = ${$papers{"psutils"}}[0];
+  $papers{"psutils"} = TeXLive::TLPaper::get_paper_list("psutils");
+  $currentpaper{"psutils"} = $papers{"psutils"}->[0];
 }
 
 
@@ -1401,6 +1399,7 @@ sub do_gui_language_setting {
     sk => "Slovak",
     sl => "Slovenian",
     sr => "Serbian",
+    uk => "Ukrainian",
     vi => "Vietnamese",
     "zh_CN" => "Simplified Chinese",
     "zh_TW" => "Traditional Chinese"
@@ -1501,7 +1500,7 @@ sub ask_one_repository {
   #$entry->pack(-side => "left",-padx => "2m", -pady => "2m");
 
   my $f2 = $sw->Frame;
-  $f2->Button(-text => __("Choose Directory"), 
+  $f2->Button(-text => __("Choose directory"), 
     -command => sub {
                       $val = $sw->chooseDirectory;
                       #if (defined($var)) {
@@ -1943,8 +1942,6 @@ sub SelectedPackages {
 sub critical_updates_done_msg_and_end {
   # terminate here immediately so that we are sure the auto-updater
   # is run immediately
-  # make sure we exit in finish(0)
-  $::gui_mode = 0;
   # warn that program will now be terminated
   $mw->Dialog(-title => __("Warning"),
     -text => __("Critical updates have been installed.\nProgram will terminate now.\nPlease restart if necessary."),
@@ -1952,7 +1949,8 @@ sub critical_updates_done_msg_and_end {
   # also delete the main window before we kill the process to 
   # make sure that Tk is happy (segfault on cmd line, email Taco)
   $mw->destroy;
-  finish(0); 
+  # don't call finish(0) as we need to exit immediately
+  exit(0); 
 }
   
 sub update_all_packages {
@@ -2060,35 +2058,6 @@ sub populate_Packages {
   $Packages{$p}{'match_desc'}    .= "\n";
   $Packages{$p}{'match_desc'}    .= ($tlp->longdesc || "");
   #
-  # taxonomy matching
-  if (defined($taxonomy)) {
-    $Packages{$p}{'taxonomy'} = "";
-    if (defined($taxonomy->{'by-package'}{'keyword'}{$p})) {
-      $Packages{$p}{'keyword'} = 
-        join(", ", @{$taxonomy->{'by-package'}{'keyword'}{$p}});
-      $Packages{$p}{'taxonomy'} .=
-        join('\n', @{$taxonomy->{'by-package'}{'keyword'}{$p}});
-      $Packages{$p}{'taxonomy'} .= '\n';
-    }
-    if (defined($taxonomy->{'by-package'}{'functionality'}{$p})) {
-      $Packages{$p}{'functionality'} =
-        join(' > ', @{$taxonomy->{'by-package'}{'functionality'}{$p}});
-      $Packages{$p}{'taxonomy'} .=
-        join('\n', @{$taxonomy->{'by-package'}{'functionality'}{$p}});
-      $Packages{$p}{'taxonomy'} .= '\n';
-    }
-    if (defined($taxonomy->{'by-package'}{'primary'}{$p})) {
-      $Packages{$p}{'primary'} = $taxonomy->{'by-package'}{'primary'}{$p};
-      $Packages{$p}{'taxonomy'} .= $taxonomy->{'by-package'}{'primary'}{$p};
-      $Packages{$p}{'taxonomy'} .= "\n";
-    }
-    if (defined($taxonomy->{'by-package'}{'secondary'}{$p})) {
-      $Packages{$p}{'secondary'} = $taxonomy->{'by-package'}{'secondary'}{$p};
-      $Packages{$p}{'taxonomy'} .= $taxonomy->{'by-package'}{'secondary'}{$p};
-      $Packages{$p}{'taxonomy'} .= "\n";
-    }
-  }
-  #
   # file matching
   my @all_f = $tlp->all_files;
   if ($tlp->relocated) { for (@all_f) { s:^$RelocPrefix/:$RelocTree/:; } }
@@ -2285,7 +2254,8 @@ sub update_list_remote {
       next if member($p, @critical);
       if (defined($Packages{$p}{'localrevision'}) &&
           defined($Packages{$p}{'remoterevision'}) &&
-          $Packages{$p}{'localrevision'} < $Packages{$p}{'remoterevision'}) {
+          $Packages{$p}{'localrevision'} < 
+            maybe_strip_last_plus($Packages{$p}{'remoterevision'})) {
         $min_action++;
       }
     }
@@ -2352,7 +2322,7 @@ sub cb_handle_restore {
   # first do the handling of the backup dir selection
   {
     my ($a, $b) = check_backupdir_selection();
-    if (!$a) {
+    if ($a != $F_OK) {
       # in all these cases we want to terminate in the non-gui mode
       my $sw = $mw->DialogBox(-title => __("Warning"), -buttons => [ __("Ok") ]);
       $sw->add("Label", -text => $b)->pack(@p_iii);
@@ -2423,7 +2393,7 @@ sub cb_handle_restore {
                       $rev = "";
                       $mw->Unbusy;
                     })->pack(@p_ii);
-  $tf->Button(-text => __("Restore all package to latest version"),
+  $tf->Button(-text => __("Restore all packages to latest version"),
     -command => sub {
                       $mw->Busy(-recurse => 1);
                       for my $p (@pkgbackup) {
@@ -2623,18 +2593,17 @@ sub update_loaded_location_string {
   my @tags = sort keys %repos;
   my @vals;
   if ($#tags > 0) {
-    @vals = map { "$_:$repos{$_}" } sort sort_main_first @tags;
+    @vals = map {
+      "$_:$repos{$_} (" .
+      ($remotetlpdb->virtual_get_tlpdb($_)->is_verified ?
+        __("verified") : __("not verified")) . ")"
+    } sort sort_main_first @tags;
   } else {
+    $arg .= " (" . ($remotetlpdb->is_verified ?
+      __("verified") : __("not verified")) . ")";
     @vals = ( $arg );
   }
-  my $locstr;
   if ($#tags > 0) {
-    $locstr = $repos{'main'};
-    for my $t (keys %repos) {
-      if ($t ne 'main') {
-        $locstr .= " $repos{$t}";
-      }
-    }
     $loaded_text_button->configure(-text => __("multiple repositories"));
   } else {
     $loaded_text_button->configure(-text => $arg);
