@@ -1,59 +1,49 @@
 #!/usr/bin/env perl
-# $Id: updmap.pl 34884 2014-08-09 12:20:53Z preining $
+# $Id: updmap.pl 42939 2017-01-13 14:32:10Z preining $
 # updmap - maintain map files for outline fonts.
 # (Maintained in TeX Live:Master/texmf-dist/scripts/texlive.)
 # 
-# Copyright 2011, 2012, 2013, 2014 Norbert Preining
+# Copyright 2011-2016 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 #
 # History:
 # Original shell script (C) 2002 Thomas Esser
-# first perl variant (C) Fabrice Popineau
+# first perl variant by Fabrice Popineau
 # later adaptions by Reinhard Kotucha and Karl Berry
 # the original versions were licensed under the following agreement:
 # Anyone may freely use, modify, and/or distribute this file, without
-# limitation.
-#
-# TODO
-# - check all other invocations
+
+my $svnid = '$Id: updmap.pl 42939 2017-01-13 14:32:10Z preining $';
 
 my $TEXMFROOT;
-
 BEGIN {
-  $^W=1;
+  $^W = 1;
   $TEXMFROOT = `kpsewhich -var-value=TEXMFROOT`;
   if ($?) {
-    print_error("Cannot find TEXMFROOT, aborting!\n");
-    exit 1;
+    die "$0: kpsewhich -var-value=TEXMFROOT failed, aborting early.\n";
   }
   chomp($TEXMFROOT);
-  unshift (@INC, "$TEXMFROOT/tlpkg");
+  unshift(@INC, "$TEXMFROOT/tlpkg");
 }
 
-
-my $svnid = '$Id: updmap.pl 34884 2014-08-09 12:20:53Z preining $';
-my $lastchdate = '$Date: 2014-08-09 14:20:53 +0200 (Sat, 09 Aug 2014) $';
+my $lastchdate = '$Date: 2017-01-13 15:32:10 +0100 (Fri, 13 Jan 2017) $';
 $lastchdate =~ s/^\$Date:\s*//;
 $lastchdate =~ s/ \(.*$//;
-my $svnrev = '$Revision: 34884 $';
+my $svnrev = '$Revision: 42939 $';
 $svnrev =~ s/^\$Revision:\s*//;
 $svnrev =~ s/\s*\$$//;
-my $version = "svn$svnrev ($lastchdate)";
+my $version = "r$svnrev ($lastchdate)";
 
 use Getopt::Long qw(:config no_autoabbrev ignore_case_always);
 use strict;
 use TeXLive::TLUtils qw(mkdirhier mktexupd win32 basename dirname 
   sort_uniq member touch);
 
-#use Data::Dumper;
-#$Data::Dumper::Indent = 1;
-
 (my $prg = basename($0)) =~ s/\.pl$//;
 
-# sudo sometimes does not reset the home dir of root, check on that
-# see more comments at the definition of the function itself
-# this function checks by itself whether it is running on windows or not
+# sudo sometimes does not reset the home dir of root;
+# see more comments at the definition of this function.
 reset_root_home();
 
 chomp(my $TEXMFDIST = `kpsewhich --var-value=TEXMFDIST`);
@@ -77,7 +67,6 @@ if (win32()) {
 my $texmfconfig = $TEXMFCONFIG;
 my $texmfvar    = $TEXMFVAR;
 
-
 my %opts = ( quiet => 0, nohash => 0, nomkmap => 0 );
 my $alldata;
 my $updLSR;
@@ -118,6 +107,7 @@ my @cmdline_options = (
   "help|h",
   # some debugging invocations
   "_readsave=s",
+  "_dump",
   );
 
 my %settings = (
@@ -146,20 +136,32 @@ my %settings = (
     type     => "binary",
     default  => "false",
   },
-  kanjiEmbed            => {
+  jaEmbed               => {
     type     => "any",
     default  => "noEmbed",
   },
-  kanjiVariant          => {
+  jaVariant             => {
     type     => "any",
     default  => "",
   },
+  scEmbed            => {
+    type     => "any",
+    default  => "noEmbed",
+  },
+  tcEmbed            => {
+    type     => "any",
+    default  => "noEmbed",
+  },
+  koEmbed            => {
+    type     => "any",
+    default  => "noEmbed",
+  },
 );
 
 &main();
 
-###############
-
+##################################################################
+#
 sub main {
   processOptions();
 
@@ -170,7 +172,6 @@ sub main {
     exit (0);
   }
 
-  #
   # check if we are in *hidden* sys mode, in which case we switch
   # to sys mode
   # Nowdays we use -sys switch instead of simply overriding TEXMFVAR
@@ -187,8 +188,6 @@ sub main {
     }
   }
 
-
-
   if ($opts{'sys'}) {
     # we are running as updmap-sys, make sure that the right tree is used
     $texmfconfig = $TEXMFSYSCONFIG;
@@ -197,12 +196,15 @@ sub main {
 
   if ($opts{'dvipdfmoutputdir'} && !defined($opts{'dvipdfmxoutputdir'})) {
     $opts{'dvipdfmxoutputdir'} = $opts{'dvipdfmoutputdir'};
-    print_warning("Using --dvipdfmoutputdir options for dvipdfmx, but please use --dvipdfmxoutputdir\n");
+    print_warning("Using --dvipdfmoutputdir options for dvipdfmx,"
+                  . " but please use --dvipdfmxoutputdir\n");
   }
 
-  if ($opts{'dvipdfmoutputdir'} && $opts{'dvipdfmxoutputdir'} &&
-      $opts{'dvipdfmoutputdir'} ne $opts{'dvipdfmxoutputdir'}) {
-    print_error("Options for --dvipdfmoutputdir and --dvipdfmxoutputdir do not agree\nplease use only --dvipdfmxoutputdir. Exiting.\n");
+  if ($opts{'dvipdfmoutputdir'} && $opts{'dvipdfmxoutputdir'}
+      && $opts{'dvipdfmoutputdir'} ne $opts{'dvipdfmxoutputdir'}) {
+    print_error("Options for --dvipdfmoutputdir and --dvipdfmxoutputdir"
+                . " do not match\n"
+                . "Please use only --dvipdfmxoutputdir; exiting.\n");
     exit(1);
   }
 
@@ -210,7 +212,6 @@ sub main {
     read_updmap_files($opts{'_readsave'});
     merge_settings_replace_kanji();
     print "READING DONE ============================\n";
-    #print Dumper($alldata);
     $alldata->{'updmap'}{$opts{'_readsave'}}{'changed'} = 1;
     save_updmap($opts{'_readsave'});
     exit 0;
@@ -308,13 +309,13 @@ sub main {
     # TEXMFLOCAL     $TEXLIVE/texmf-local/web2c/updmap.cfg
     # TEXMFDIST      $TEXLIVE/YYYY/texmf-dist/web2c/updmap.cfg
     #
-    @{$opts{'cnffile'}}  = @used_files;
+    @{$opts{'cnffile'}} = @used_files;
     #
-    # determine the config file that we will use for changes
-    # if in the list of used files contains either one from
+    # Determine the config file that we will use for changes:
+    # if the list of used files contains one from either
     # TEXMFHOME or TEXMFCONFIG (which is TEXMFSYSCONFIG in the -sys case)
-    # then use the *top* file (which will be either one of the two),
-    # if none of the two exists, create a file in TEXMFCONFIG and use it
+    # then use the *top* file (which will be one of the two *CONFIG);
+    # if neither of those two exists, create a file in TEXMFCONFIG and use it.
     my $use_top = 0;
     for my $f (@used_files) {
       if ($f =~ m!(\Q$TEXMFHOME\E|\Q$texmfconfig\E)/web2c/updmap.cfg!) {
@@ -331,11 +332,11 @@ sub main {
     }
   }
   if (!$opts{'quiet'}) {
-    print "$prg is using the following updmap.cfg files (in precedence order):\n";
+    print "$prg will read the following updmap.cfg files (in precedence order):\n";
     for my $f (@{$opts{'cnffile'}}) {
       print "  $f\n";
     }
-    print "$prg is using the following updmap.cfg file for writing changes:\n";
+    print "$prg may write changes to the following updmap.cfg file:\n";
     print "  $changes_config_file\n";
   }
   if ($opts{'listfiles'}) {
@@ -347,6 +348,18 @@ sub main {
 
   read_updmap_files(@{$opts{'cnffile'}});
 
+  if ($opts{'_dump'}) {
+    merge_settings_replace_kanji();
+    read_map_files();
+    require Data::Dumper;
+    # two times to silence perl warnings!
+    $Data::Dumper::Indent = 1;
+    $Data::Dumper::Indent = 1;
+    print "READING DONE ============================\n";
+    print Data::Dumper::Dumper($alldata);
+    exit 0;
+  }
+
   if ($opts{'showoption'}) {
     merge_settings_replace_kanji();
     for my $o (@{$opts{'showoption'}}) {
@@ -361,12 +374,22 @@ sub main {
     exit 0;
   }
 
-  if ($opts{'listmaps'}) {
+  if ($opts{'listmaps'} || $opts{'listavailablemaps'}) {
     merge_settings_replace_kanji();
-    for my $m (keys %{$alldata->{'maps'}}) {
+    # only check for missing map files 
+    # (pass in true argument to read_map_files)
+    my %missing = map { $_ => 1 } read_map_files(1);
+    for my $m (sort keys %{$alldata->{'maps'}}) {
+      next if ($missing{$m} && $opts{'listavailablemaps'});
       my $origin = $alldata->{'maps'}{$m}{'origin'};
-      print $alldata->{'updmap'}{$origin}{'maps'}{$m}{'type'}, " $m ",
-      $alldata->{'updmap'}{$origin}{'maps'}{$m}{'status'}, " in $origin\n";
+      my $type = ($origin eq 'builtin' ? 'Map' :
+        $alldata->{'updmap'}{$origin}{'maps'}{$m}{'type'});
+      my $status = ($origin eq 'builtin' ? 'enabled' :
+        $alldata->{'updmap'}{$origin}{'maps'}{$m}{'status'});
+      my $avail = ($missing{$m} ? "\t(not available)" : '');
+      print "$type\t$m\t$status\t$origin$avail\n";
+      #print $alldata->{'updmap'}{$origin}{'maps'}{$m}{'type'}, " $m ",
+      #$alldata->{'updmap'}{$origin}{'maps'}{$m}{'status'}, " in $origin\n";
     }
     exit 0;
   }
@@ -484,8 +507,6 @@ sub main {
 
 ##################################################################
 #
-#
-
 sub getFonts {
   my ($first, @rest) = @_;
   my $getall = 0;
@@ -803,8 +824,6 @@ sub transLW35 {
 #       ... %!PS fontname %other comments
 #       ... %!PS fontname
 #
-###############################################################################
-
 # reimplementation of the cryptic code that was there before
 sub cidx2dvips {
   my ($s) = @_;
@@ -988,8 +1007,19 @@ sub mkMaps {
   my ($pdftexDownloadBase14, $pdftexDownloadBase14_origin) = 
     get_cfg('pdftexDownloadBase14');
   my ($pxdviUse, $pxdviUse_origin) = get_cfg('pxdviUse');
-  my ($kanjiEmbed, $kanjiEmbed_origin) = get_cfg('kanjiEmbed');
-  my ($kanjiVariant, $kanjiVariant_origin) = get_cfg('kanjiVariant');
+  my ($jaEmbed, $jaEmbed_origin) = get_cfg('jaEmbed');
+  my ($jaVariant, $jaVariant_origin) = get_cfg('jaVariant');
+  my ($scEmbed, $scEmbed_origin) = get_cfg('scEmbed');
+  my ($tcEmbed, $tcEmbed_origin) = get_cfg('tcEmbed');
+  my ($koEmbed, $koEmbed_origin) = get_cfg('koEmbed');
+
+  # keep backward compatibility with old definitions
+  # of kanjiEmbed, kanjiVariant
+  ($jaEmbed, $jaEmbed_origin) = get_cfg('kanjiEmbed')
+    if (!defined($jaEmbed));
+  ($jaVariant, $jaVariant_origin) = get_cfg('kanjiVariant')
+    if (!defined($jaVariant));
+
 
   # pxdvi is optional, and off by default.  Don't create the output
   # directory unless we are going to put something there.
@@ -1007,10 +1037,16 @@ sub mkMaps {
          .      "$dvipsDownloadBase35 ($dvipsDownloadBase35_origin)"
          . "\n  download standard fonts (pdftex) : "
          .      "$pdftexDownloadBase14 ($pdftexDownloadBase14_origin)"
-         . "\n  kanjiEmbed replacement string    : "
-         .      "$kanjiEmbed ($kanjiEmbed_origin)"
-         . "\n  kanjiVariant replacement string  : "
-         .      "$kanjiVariant ($kanjiVariant_origin)"
+         . "\n  jaEmbed replacement string       : "
+         .      "$jaEmbed ($jaEmbed_origin)"
+         . "\n  jaVariant replacement string     : "
+         .      ($jaVariant ? $jaVariant : "<empty>") . " ($jaVariant_origin)"
+         . "\n  scEmbed replacement string       : "
+         .      "$scEmbed ($scEmbed_origin)"
+         . "\n  tcEmbed replacement string       : "
+         .      "$tcEmbed ($tcEmbed_origin)"
+         . "\n  koEmbed replacement string       : "
+         .      "$koEmbed ($koEmbed_origin)"
          . "\n  create a mapfile for pxdvi       : "
          .      "$pxdviUse ($pxdviUse_origin)"
          . "\n\n");
@@ -1032,6 +1068,7 @@ sub mkMaps {
   for my $m (keys %{$alldata->{'maps'}}) {
     my $origin = $alldata->{'maps'}{$m}{'origin'};
     next if !defined($origin);
+    next if ($origin eq 'builtin');
     next if ($alldata->{'updmap'}{$origin}{'maps'}{$m}{'status'} eq "disabled");
     push @mixedmaps, $m
       if ($alldata->{'updmap'}{$origin}{'maps'}{$m}{'type'} eq "MixedMap");
@@ -1128,7 +1165,7 @@ sub mkMaps {
 
   if ($pxdviUse eq "true") {
     # we use the very same data as for kanjix.map, but generate
-    # a different file, so that in case a user wants to hand-craft it
+    # a different file, in case a user wants to hand-craft it
     print "Generating output for pxdvi...\n" if !$opts{'quiet'};
      &writeLines(">$pxdvioutputdir/xdvi-ptex.map", @kanjimaps_fonts);
   }
@@ -1449,7 +1486,6 @@ sub enable_disable_maps {
   my $tc = $alldata->{'changes_config'};
   die "$prg: top config file $tc has not been read."
     if (!defined($alldata->{'updmap'}{$tc}));
-  my $changed = 0;
 
   for my $w (@what) {
     if ($w =~ m/=/) {
@@ -1632,6 +1668,16 @@ sub check_option {
 sub setOption {
   my ($opt, $val) = @_;
 
+  # allow backward compatility with old kanjiEmbed and kanjiVariant settings
+  if ($opt eq "kanjiEmbed") {
+    print_warning("using jaEmbed instead of kanjiEmbed\n");
+    $opt = "jaEmbed";
+  }
+  if ($opt eq "kanjiVariant") {
+    print_warning("using jaVariant instead of kanjiVariant\n");
+    $opt = "jaVariant";
+  }
+
   die "$prg: Unsupported option $opt." if (!defined($settings{$opt}));
   die "$0: Invalid value $val for option $opt." 
     if (!check_option($opt, $val));
@@ -1855,8 +1901,19 @@ sub merge_settings_replace_kanji {
     }
   }
   #
-  my ($kanjiEmbed, $kanjiEmbed_origin) = get_cfg('kanjiEmbed');
-  my ($kanjiVariant, $kanjiVariant_origin) = get_cfg('kanjiVariant');
+  my ($jaEmbed, $jaEmbed_origin) = get_cfg('jaEmbed');
+  my ($jaVariant, $jaVariant_origin) = get_cfg('jaVariant');
+  my ($scEmbed, $scEmbed_origin) = get_cfg('scEmbed');
+  my ($tcEmbed, $tcEmbed_origin) = get_cfg('tcEmbed');
+  my ($koEmbed, $koEmbed_origin) = get_cfg('koEmbed');
+
+  # keep backward compatibility with old definitions
+  # of kanjiEmbed, kanjiVariant
+  ($jaEmbed, $jaEmbed_origin) = get_cfg('kanjiEmbed')
+    if (!defined($jaEmbed));
+  ($jaVariant, $jaVariant_origin) = get_cfg('kanjiVariant')
+    if (!defined($jaVariant));
+
   #
   # go through all map files and check that the text is properly replaced
   # after the replacement check that the generated map file actually
@@ -1864,10 +1921,19 @@ sub merge_settings_replace_kanji {
   #
   for my $l (@l) {
     for my $m (keys %{$alldata->{'updmap'}{$l}{'maps'}}) {
-      if ($m =~ m/\@kanjiEmbed@/ || $m =~ m/\@kanjiVariant@/) {
-        my $newm = $m;
-        $newm =~ s/\@kanjiEmbed@/$kanjiEmbed/;
-        $newm =~ s/\@kanjiVariant@/$kanjiVariant/;
+      my $newm = $m;
+      # do all kinds of substitutions
+      $newm =~ s/\@jaEmbed@/$jaEmbed/;
+      $newm =~ s/\@jaVariant@/$jaVariant/;
+      $newm =~ s/\@scEmbed@/$scEmbed/;
+      $newm =~ s/\@tcEmbed@/$tcEmbed/;
+      $newm =~ s/\@koEmbed@/$koEmbed/;
+      # also do substitutions of old strings in case they are left
+      # over somewhere
+      $newm =~ s/\@kanjiEmbed@/$jaEmbed/;
+      $newm =~ s/\@kanjiVariant@/$jaVariant/;
+      if ($newm ne $m) {
+        # something was substituted
         if (locateMap($newm)) {
           # now we have to update various linked items
           $alldata->{'updmap'}{$l}{'maps'}{$newm}{'type'} =
@@ -1878,7 +1944,7 @@ sub merge_settings_replace_kanji {
             $alldata->{'updmap'}{$l}{'maps'}{$m}{'line'};
           $alldata->{'updmap'}{$l}{'maps'}{$newm}{'original'} = $m;
         } else {
-          print_warning("generated map $newm (from $m) does not exists, not activating it!\n");
+          print_warning("generated map $newm (from $m) does not exist, not activating it!\n");
         }
         # in any case delete the @kanji...@ entry line, such a map will
         # never exist
@@ -1945,6 +2011,9 @@ sub read_updmap_file {
     if (@rest) {
       print_warning("line $i in $fn contains a syntax error, more than two words!\n");
     }
+    # backward compatibility with kanjiEmbed/kanjiVariant
+    $a = ($a eq "kanjiEmbed" ? "jaEmbed" : $a);
+    $a = ($a eq "kanjiVariant" ? "jaVariant" : $a);
     if (defined($settings{$a})) {
       if (check_option($a, $b)) {
         $data{'setting'}{$a}{'val'} = $b;
@@ -1968,6 +2037,7 @@ sub read_updmap_file {
 }
 
 sub read_map_files {
+  my $quick = shift;
   if (!defined($alldata->{'updmap'})) {
     return;
   }
@@ -1986,6 +2056,7 @@ sub read_map_files {
   for my $m (qw/dvips35.map pdftex35.map ps2pk35.map/) {
     push @maps, $m;
     $alldata->{'maps'}{$m}{'status'} = 'enabled';
+    $alldata->{'maps'}{$m}{'origin'} = 'builtin';
   }
   @maps = sort_uniq(@maps);
   my @fullpath = `kpsewhich --format=map @maps`;
@@ -2000,6 +2071,7 @@ sub read_map_files {
       push @missing, $map;
     }
   }
+  return @missing if $quick;
 
   #
   # read in the three basic fonts definition maps
@@ -2093,6 +2165,7 @@ sub merge_data {
   for my $m (keys %{$alldata->{'maps'}}) {
     my $origin = $alldata->{'maps'}{$m}{'origin'};
     next if !defined($origin);
+    next if ($origin eq 'builtin');
     next if ($alldata->{'updmap'}{$origin}{'maps'}{$m}{'status'} eq "disabled");
     for my $f (keys %{$alldata->{'maps'}{$m}{'fonts'}}) {
       # use the font definition only for those fonts where the origin matches
@@ -2170,13 +2243,14 @@ sub help {
 Usage: $prg     [OPTION] ... [COMMAND]
    or: $prg-sys [OPTION] ... [COMMAND]
 
-Update the default font map files used by pdftex, dvips, and dvipdfm(x),
-and optionally pxdvi, as determined by all configuration files updmap.cfg 
-(the ones returned by running "kpsewhich --all updmap.cfg", but see below).
+Update the default font map files used by pdftex (pdftex.map), dvips
+(psfonts.map), and dvipdfm(x), and optionally pxdvi, as determined by
+all configuration files updmap.cfg (the ones returned by running
+"kpsewhich --all updmap.cfg", but see below).
 
 Among other things, these map files are used to determine which fonts
 should be used as bitmaps and which as outlines, and to determine which
-font files are included in the PDF or PostScript output.
+font files are included, typically subsetted, in the PDF or PostScript output.
 
 updmap-sys is intended to affect the system-wide configuration, while
 updmap affects personal configuration files only, overriding the system
@@ -2186,6 +2260,20 @@ warning in this situation.)
 
 By default, the TeX filename database (ls-R) is also updated.
 
+The updmap system is regrettably complicated, for both inherent and
+historical reasons.  A general overview:
+- updmap.cfg files are mainly about listing other files, namely the
+  font-specific .maps, in which each line gives information about a
+  different TeX (.tfm) font.
+- updmap reads the updmap.cfg files and then concatenates the
+  contents of those .map files into the main output files: psfonts.map
+  for dvips and pdftex.map for pdftex and dvipdfmx.
+- The updmap.cfg files themselves are created and updated at package
+  installation time, by the system installer or the package manager or
+  by hand, and not (by default) by updmap.
+
+Good luck.
+
 Options:
   --cnffile FILE            read FILE for the updmap configuration 
                              (can be given multiple times, in which case
@@ -2216,17 +2304,43 @@ Commands:
   --enable MixedMap=MAPFILE add \"MixedMap MAPFILE\" to updmap.cfg
   --enable KanjiMap=MAPFILE add \"KanjiMap MAPFILE\" to updmap.cfg
   --disable MAPFILE         disable MAPFILE, of whatever type
-  --listmaps                list all active and inactive maps
-  --listavailablemaps       same as --listmaps, but without
-                             unavailable map files
+  --listmaps                list all maps (details below)
+  --listavailablemaps       list available maps (details below)
   --syncwithtrees           disable unavailable map files in updmap.cfg
 
-Explanation of the map types: the (only) difference between Map and
-MixedMap is that MixedMap entries are not added to psfonts_pk.map.
-The purpose is to help users with devices that render Type 1 outline
-fonts worse than mode-tuned Type 1 bitmap fonts.  So, MixedMap is used
-for fonts that are available as both Type 1 and Metafont.
-KanjiMap entries are added to psfonts_t1.map and kanjix.map.
+The main output:
+
+  The main output of updmap is the files containing the individual font
+  map lines which the drivers (dvips, pdftex, etc.) read to handle fonts.
+  
+  The map files for dvips (psfonts.map) and pdftex and dvipdfmx
+  (pdftex.map) are written to TEXMFVAR/fonts/map/updmap/{dvips,pdftex}/.
+  
+  In addition, information about Kanji fonts is written to
+  TEXMFVAR/fonts/map/updmap/dvipdfmx/kanjix.map, and optionally to 
+  TEXMFVAR/fonts/map/updmap/pxdvi/xdvi-ptex.map.  These are for Kanji
+  only and are not like other map files.  dvipdfmx reads pdftex.map for
+  the map entries for non-Kanji fonts.
+  
+  If no option is given, so the invocation is just "updmap" or
+  "updmap-sys", these output files are always recreated.
+
+  Otherwise, if an option such as --enable or --disable is given, the
+  output files are recreated if the list of enabled map files (from
+  updmap.cfg) has changed.  The --force option overrides this,
+  always recreating the output files.
+  
+Explanation of the map types:
+
+  The normal type is Map.
+  
+  The only difference between Map and MixedMap is that MixedMap entries
+  are not added to psfonts_pk.map.  The purpose is to help users with
+  devices that render Type 1 outline fonts worse than mode-tuned Type 1
+  bitmap fonts.  So, MixedMap is used for fonts that are available as
+  both Type 1 and Metafont.
+
+  KanjiMap entries are added to psfonts_t1.map and kanjix.map.
 
 Explanation of the OPTION names for --showoptions, --showoption, --setoption:
 
@@ -2238,8 +2352,11 @@ Explanation of the OPTION names for --showoptions, --showoption, --setoption:
     Whether pdftex includes the standard 14 PDF fonts in its output.
   pxdviUse              true,false  (default false)
     Whether maps for pxdvi (Japanese-patched xdvi) are under updmap's control.
-  kanjiEmbed            (any string)
-  kanjiVariant          (any string)
+  jaEmbed               (any string)
+  jaVariant             (any string)
+  scEmbed               (any string)
+  tcEmbed               (any string)
+  koEmbed               (any string)
     See below.
   LW35                  URWkb,URW,ADOBEkb,ADOBE  (default URWkb)
     Adapt the font and file names of the standard 35 PostScript fonts.
@@ -2255,20 +2372,23 @@ Explanation of the OPTION names for --showoptions, --showoption, --setoption:
   command-line options or configuration files to the programs, as
   explained at the beginning of updmap.cfg.
 
-  The options kanjiEmbed and kanjiVariant specify special replacements
-  in the map lines.  If a map contains the string \@kanjiEmbed\@, then
-  this will be replaced by the value of that option; similarly for
-  kanjiVariant.  In this way, users of Japanese TeX can select different
-  fonts to be included in the final output.
+  The options jaEmbed and jaVariant (formerly kanjiEmbed and kanjiVariant)
+  specify special replacements in the map lines.  If a map contains the 
+  string \@jaEmbed\@, then this will be replaced by the value of that option;
+  similarly for jaVariant.  In this way, users of Japanese TeX can select
+  different fonts to be included in the final output.  The counterpart for
+  Simplified Chinese, Traditional Chinese and Korean fonts are
+  scEmbed, tcEmbed and koEmbed respectively.
 
 Explanation of trees and files normally used:
 
-  If --cnffile is specified on the command line (possibly multiple
-  times), its value(s) are used.  Otherwise, updmap reads all the
-  updmap.cfg files found by running \`kpsewhich -all updmap.cfg', in the
-  order returned by kpsewhich.
+  If --cnffile is specified on the command line (can be given multiple
+  times), its value(s) is(are) used.  Otherwise, updmap reads all the
+  updmap.cfg files found by running \`kpsewhich -all updmap.cfg',
+  in the order returned by kpsewhich (which is the order of trees
+  defined in texmf.cnf).
 
-  In any case, if multiple updmap.cfg files are found, all the maps
+  In either case, if multiple updmap.cfg files are found, all the maps
   mentioned in all the updmap.cfg files are merged.
 
   Thus, if updmap.cfg files are present in all trees, and the default
@@ -2295,25 +2415,31 @@ Explanation of trees and files normally used:
   According to the actions, updmap might write to one of the given files
   or create a new updmap.cfg, described further below.
 
-Where changes are saved: 
+Where and which updmap.cfg changes are saved: 
 
-  If config files are given on the command line, then the first one 
-  given will be used to save any changes from --setoption, --enable 
-  or --disable.  If the config files are taken from kpsewhich output, 
-  then the algorithm is more complex:
+  When no options are given, the updmap.cfg file(s) are only read, not
+  written.  It's when an option --setoption, --enable or --disable is
+  specified that an updmap.cfg needs to be updated.  In this case:
 
-    1) If \$TEXMFCONFIG/web2c/updmap.cfg or \$TEXMFHOME/web2c/updmap.cfg
+  1) If config files are given on the command line, then the first one
+  given is used to save any such changes.
+  
+  2) If the config files are taken from kpsewhich output, then the
+  algorithm is more complex:
+
+    2a) If \$TEXMFCONFIG/web2c/updmap.cfg or \$TEXMFHOME/web2c/updmap.cfg
     appears in the list of used files, then the one listed first by
     kpsewhich --all (equivalently, the one returned by kpsewhich
     updmap.cfg), is used.
       
-    2) If neither of the above two are present and changes are made, a
+    2b) If neither of the above two are present and changes are made, a
     new config file is created in \$TEXMFCONFIG/web2c/updmap.cfg.
   
-  In general, the idea is that if a given config file is not writable, a
-  higher-level one can be used.  That way, the distribution's settings
-  can be overridden for system-wide using TEXMFLOCAL, and then system
-  settings can be overridden again for a particular using using TEXMFHOME.
+  In general, the idea is that if the user cannot write to a given
+  config file, a higher-level one can be used.  That way, the
+  distribution's settings can be overridden system-wide using
+  TEXMFLOCAL, and system settings can be overridden again in a
+  particular user's TEXMFHOME.
 
 Resolving multiple definitions of a font:
 
@@ -2330,7 +2456,9 @@ Disabling maps:
     \#! Map mapname.map
   or
     \#! MixedMap mapname.map
-  in the higher-priority updmap.cfg file. 
+  in the higher-priority updmap.cfg file.  (The \#! must be at the
+  beginning of the line, with at least one space or tab afterward, and
+  whitespace between each word on the list.)
 
   As an example, suppose you have a copy of MathTime Pro fonts
   and want to disable the Belleek version of the fonts; that is,
@@ -2341,27 +2469,39 @@ Disabling maps:
     Map mt-yy.map
   and call $prg.
 
-updmap writes the map files for dvips (psfonts.map) and pdftex
-(pdftex.map) to the TEXMFVAR/fonts/map/updmap/{dvips,pdftex}/
-directories.   
+Listing of maps:
 
-The log file is written to TEXMFVAR/web2c/updmap.log.
+  The two options --listmaps and --listavailablemaps list all maps
+  defined in any of the updmap.cfg files (for --listmaps), and 
+  only those actually found on the system (for --listavailablemaps).
+  The output format is one line per font map, with the following
+  fields separated by tabs: map, type (Map, MixedMap, KanjiMap),
+  status (enabled, disabled), origin (the updmap.cfg file where
+  it is mentioned, or 'builtin' for the three basic maps).
 
-When updmap-sys is run, TEXMFSYSCONFIG and TEXMFSYSVAR are used
-instead of TEXMFCONFIG and TEXMFVAR, respectively.  This is the only
-difference between updmap-sys and updmap.
+  In the case of --listmaps there can be one additional fields
+  (again separated by tab) containing '(not available)' for those
+  map files that cannot be found.
+ 
+updmap vs. updmap-sys:
 
-Other locations may be used if you give them on the command line, or
-these trees don't exist, or you are not using the original TeX Live.
+  When updmap-sys is run, TEXMFSYSCONFIG and TEXMFSYSVAR are used
+  instead of TEXMFCONFIG and TEXMFVAR, respectively.  This is the
+  primary difference between updmap-sys and updmap.
+
+  Other locations may be used if you give them on the command line, or
+  these trees don't exist, or you are not using the original TeX Live.
 
 To see the precise locations of the various files that
 will be read and written, give the -n option (or read the source).
 
+The log file is written to TEXMFVAR/web2c/updmap.log.
+
 For step-by-step instructions on making new fonts known to TeX, read
 http://tug.org/fonts/fontinstall.html.  For even more terse
-instructions, read the beginning of the main updmap.cfg.
+instructions, read the beginning of the main updmap.cfg file.
 
-Report bugs to: tex-k\@tug.org
+Report bugs to: tex-live\@tug.org
 TeX Live home page: <http://tug.org/texlive/>
 EOF
 ;
