@@ -37,9 +37,20 @@ class Language
 		@licence = language_hash["licence"]
 		@authors = language_hash["authors"]
 
+		@has_quotes = false
+		@has_dashes = false
+
 		if @synonyms==nil then @synonyms = [] end
 	end
 
+	def set_quotes
+		@has_quotes = true
+	end
+
+	def set_dashes
+		@has_dashes = true
+	end
+
 	# TODO: simplify this (reduce duplication)
 
 	def get_exceptions
@@ -113,6 +124,7 @@ class Language
 	attr_reader :code, :name, :synonyms, :hyphenmin, :encoding, :exceptions, :message
 	attr_reader :description_s, :description_l, :version
 	attr_reader :licence, :authors
+	attr_reader :has_quotes, :has_dashes
 	# this hack is needed for Serbian
 	attr_writer :code
 end
@@ -205,6 +217,8 @@ class Authors < Hash
 	# email doesn't work
 	"javier_mugica"       => ["Javier", "Múgica", "javier{at}digi21{dot}eu", true, true],
 	"georgi_boshnakov"    => ["Georgi", "Boshnakov", "georgi{dot}boshnakov{at}manchester{dot}ac{dot}uk", true, true],
+	"mike_kroutikov"      => ["Mike", "Kroutikov", "pgmmpk{at}gmail{dot}com", true, true],
+	"aleksandr_andreev"   => ["Aleksandr", "Andreev", "", true, true],
 }
 #
 		authors.each do |a|
@@ -384,7 +398,7 @@ class Languages < Hash
 	"message" => "Hyphenation patterns for Ancient Greek",
 
 	"version"       => "5.0",
-	"last_modified" => "2008-06-06",
+	"last_modified" => "2016-05-12",
 	"type"          => "rules",
 	"authors"       => ["dimitrios_filippou"],
 	"licence"       => "LPPL",
@@ -828,18 +842,17 @@ class Languages < Hash
 	"use_new_loader" => true,
 	"use_old_patterns" => false,
 	"filename_old_patterns" => "frhyph.tex",
-	"hyphenmin" => [2,3],
+	"hyphenmin" => [2,2],
 	"encoding" => "ec",
 	"exceptions" => false,
-	"message" => "French hyphenation patterns (V2.12, 2002/12/11)",
+	"message" => "French hyphenation patterns",
 
-	"version"       => "2.12",
-	"last_modified" => "2002-12-11",
+	"version"       => "2.13",
+	"last_modified" => "2016-05-12",
 	"type"          => "rules",
-	"authors"       => ["rene_bastian", "daniel_flipo", "bernard_gaulle"],
-	# TODO for Arthur: recreate mailing-list
-	# "email"         => ["cesure-l{at}gutenberg{dot}eu{dot}org"],
-	"licence"       => "other-free", # Knuthian type
+	"authors"       => ["daniel_flipo", "bernard_gaulle", "arthur_reutenauer"],
+	"email"         => ["cesure-l{at}gutenberg{dot}eu{dot}org"],
+	"licence"       => "MIT",
 	"description_s" => "French hyphenation patterns",
 	"description_l" => [
 		#......................................................................#
@@ -1268,6 +1281,30 @@ class Languages < Hash
 		"those of 'plain' Latin, the latter being more adapted to modern Latin.",
 	],
 },
+# liturgicallatin
+{
+	"code" => "la-x-liturgic",
+	"name" => "liturgicallatin",
+	"use_new_loader" => true,
+	"use_old_patterns" => false,
+	"filename_old_patterns" => nil,
+	"hyphenmin" => [2,2],
+	"encoding" => "ec",
+	"exceptions" => false,
+	"message" => "Liturgical Latin hyphenation patterns",
+
+	"version"       => "1.030",
+	"last_modified" => "2016-04-28",
+	"type"          => "rules",
+	"authors"       => [ "claudio_beccari" ],
+	"licence"       => "MIT",
+	"description_s" => "Liturgical Latin hyphenation patterns",
+	"description_l" => [
+		#......................................................................#
+		"Hyphenation patterns for the Liturgical Latin in T1/EC and UTF-8",
+		"encodings.",
+	],
+},
 # lithuanian
 {
 	"code" => "lt",
@@ -2071,6 +2108,26 @@ class Languages < Hash
 		"Hyphenation patterns for Georgian in T8M, T8K and UTF-8 encodings.",
 	],
 },
+# Church Slavonic
+{
+	"code" => "cu",
+	"name" => "churchslavonic",
+	"use_new_loader" => true,
+	"hyphenmin" => [1, 2],
+	"encoding" => nil,
+	"exceptions" => true,
+	"message" => "Church Slavonic hyphenation patterns",
+
+	"version" => nil,
+	"last_modified" => "2016-04-16",
+	"type" => "machine learning",
+	"authors" => ["mike_kroutikov", "aleksandr_andreev"],
+	"licence" => "MIT",
+	"description_s" => "Church Slavonic hyphenation patterns",
+	"description_l" => [
+		"Hyphenation patterns for Church Slavonic in UTF-8 encoding",
+	],
+}
 # dumylang -> dumyhyph.tex
 # nohyphenation -> zerohyph.tex
 # arabic -> zerohyph.tex
@@ -2078,8 +2135,18 @@ class Languages < Hash
 # =persian
 		]
 
+		# TODO: do not hardcode this list; auto-generate it instead
+		languages_with_quotes = ['af', 'fr', 'fur', 'it', 'oc', 'pms', 'rm', 'uk', 'zh-latn-pinyin']
+		languages_with_dashes = ['af', 'pt', 'ru', 'tk', 'uk']
+
 		languages.each do |l|
 			language = Language.new(l)
+			if languages_with_quotes.include?(language.code) then
+				language.set_quotes()
+			end
+			if languages_with_dashes.include?(language.code) then
+				language.set_dashes()
+			end
 			@@list.push(language)
 			self[language.code] = language
 		end
