@@ -10,7 +10,6 @@ local tonumber, next, type = tonumber, next, type
 
 local match, format, find, concat, gsub, lower = string.match, string.format, string.find, table.concat, string.gsub, string.lower
 local P, R, S, C, Ct, Cc, lpegmatch = lpeg.P, lpeg.R, lpeg.S, lpeg.C, lpeg.Ct, lpeg.Cc, lpeg.match
-local utfbyte = utf.byte
 local floor = math.floor
 local formatters = string.formatters
 
@@ -19,6 +18,8 @@ local trace_mapping = false  trackers.register("fonts.mapping", function(v) trac
 
 local report_fonts  = logs.reporter("fonts","loading") -- not otf only
 
+local force_ligatures = false  directives.register("fonts.mapping.forceligatures",function(v) force_ligatures = v end)
+
 local fonts         = fonts or { }
 local mappings      = fonts.mappings or { }
 fonts.mappings      = mappings
@@ -43,7 +44,7 @@ of obsolete. Some code may move to runtime or auxiliary modules.</p>
 --     end
 -- end
 
-local hex     = R("AF","09")
+local hex     = R("AF","af","09")
 ----- hexfour = (hex*hex*hex*hex)         / function(s) return tonumber(s,16) end
 ----- hexsix  = (hex*hex*hex*hex*hex*hex) / function(s) return tonumber(s,16) end
 local hexfour = (hex*hex*hex^-2) / function(s) return tonumber(s,16) end
@@ -80,12 +81,11 @@ local f_double = formatters["%04X%04X"]
 
 -- local tounicodes = table.setmetatableindex(function(t,unicode)
 --     local s
---     if unicode < 0x10000 then
+--     if unicode < 0xD7FF or (unicode > 0xDFFF and unicode <= 0xFFFF) then
 --         s = f_single(unicode)
---     elseif unicode < 0x1FFFFFFFFF then
---         s = f_double(floor(unicode/1024),unicode%1024+0xDC00)
 --     else
---         s = false
+--         unicode = unicode - 0x10000
+--         s = f_double(floor(unicode/1024)+0xD800,unicode%1024+0xDC00)
 --     end
 --     t[unicode] = s
 --     return s
@@ -139,27 +139,24 @@ local f_double = formatters["%04X%04X"]
 --     end
 -- end
 
-local function tounicode16(unicode,name)
-    if unicode < 0x10000 then
+local function tounicode16(unicode)
+    if unicode < 0xD7FF or (unicode > 0xDFFF and unicode <= 0xFFFF) then
         return f_single(unicode)
-    elseif unicode < 0x1FFFFFFFFF then
-        return f_double(floor(unicode/1024),unicode%1024+0xDC00)
     else
-        report_fonts("can't convert %a in %a into tounicode",unicode,name)
+        unicode = unicode - 0x10000
+        return f_double(floor(unicode/1024)+0xD800,unicode%1024+0xDC00)
     end
 end
 
-local function tounicode16sequence(unicodes,name)
+local function tounicode16sequence(unicodes)
     local t = { }
     for l=1,#unicodes do
         local u = unicodes[l]
-        if u < 0x10000 then
+        if u < 0xD7FF or (u > 0xDFFF and u <= 0xFFFF) then
             t[l] = f_single(u)
-        elseif unicode < 0x1FFFFFFFFF then
-            t[l] = f_double(floor(u/1024),u%1024+0xDC00)
         else
-            report_fonts ("can't convert %a in %a into tounicode",u,name)
-            return
+            u = u - 0x10000
+            t[l] = f_double(floor(u/1024)+0xD800,u%1024+0xDC00)
         end
     end
     return concat(t)
@@ -170,23 +167,20 @@ local function tounicode(unicode,name)
         local t = { }
         for l=1,#unicode do
             local u = unicode[l]
-            if u < 0x10000 then
+            if u < 0xD7FF or (u > 0xDFFF and u <= 0xFFFF) then
                 t[l] = f_single(u)
-            elseif u < 0x1FFFFFFFFF then
-                t[l] = f_double(floor(u/1024),u%1024+0xDC00)
             else
-                report_fonts ("can't convert %a in %a into tounicode",u,name)
-                return
+                u = u - 0x10000
+                t[l] = f_double(floor(u/1024)+0xD800,u%1024+0xDC00)
             end
         end
         return concat(t)
     else
-        if unicode < 0x10000 then
+        if unicode < 0xD7FF or (unicode > 0xDFFF and unicode <= 0xFFFF) then
             return f_single(unicode)
-        elseif unicode < 0x1FFFFFFFFF then
-            return f_double(floor(unicode/1024),unicode%1024+0xDC00)
         else
-            report_fonts("can't convert %a in %a into tounicode",unicode,name)
+            unicode = unicode - 0x10000
+            return f_double(floor(unicode/1024)+0xD800,unicode%1024+0xDC00)
         end
     end
 end
@@ -196,7 +190,8 @@ local function fromunicode16(str)
         return tonumber(str,16)
     else
         local l, r = match(str,"(....)(....)")
-        return (tonumber(l,16))*0x400  + tonumber(r,16) - 0xDC00
+     -- return (tonumber(l,16))*0x400  + tonumber(r,16) - 0xDC00
+        return 0x10000 + (tonumber(l,16)-0xD800)*0x400  + tonumber(r,16) - 0xDC00
     end
 end
 
@@ -214,26 +209,6 @@ end
 --     return lpegmatch(p,str)
 -- end
 
--- This is quite a bit faster but at the cost of some memory but if we
--- do this we will also use it elsewhere so let's not follow this route
--- now. I might use this method in the plain variant (no caching there)
--- but then I need a flag that distinguishes between code branches.
---
--- local cache = { }
---
--- function mappings.tounicode16(unicode)
---     local s = cache[unicode]
---     if not s then
---         if unicode < 0x10000 then
---             s = format("%04X",unicode)
---         else
---             s = format("%04X%04X",unicode/0x400+0xD800,unicode%0x400+0xDC00)
---         end
---         cache[unicode] = s
---     end
---     return s
--- end
-
 mappings.makenameparser      = makenameparser
 mappings.tounicode           = tounicode
 mappings.tounicode16         = tounicode16
@@ -319,151 +294,187 @@ function mappings.addtounicode(data,filename,checklookups)
     local ns            = 0
     local nl            = 0
     --
-    for unic, glyph in next, descriptions do
+    for du, glyph in next, descriptions do
         local name = glyph.name
         if name then
-            local index = glyph.index
-            local r = overloads[name]
-            if r then
+            local overload = overloads[name]
+            if overload then
                 -- get rid of weird ligatures
-             -- glyph.name    = r.name
-                glyph.unicode = r.unicode
-            elseif not unic or unic == -1 or unic >= private or (unic >= 0xE000 and unic <= 0xF8FF) or unic == 0xFFFE or unic == 0xFFFF then
-                local unicode = unicodevector[name] or contextvector[name]
-                if unicode then
-                    glyph.unicode = unicode
-                    ns            = ns + 1
-                end
-                -- cidmap heuristics, beware, there is no guarantee for a match unless
-                -- the chain resolves
-                if (not unicode) and usedmap then
-                    local foundindex = lpegmatch(oparser,name)
-                    if foundindex then
-                        unicode = cidcodes[foundindex] -- name to number
-                        if unicode then
-                            glyph.unicode = unicode
-                            ns            = ns + 1
-                        else
-                            local reference = cidnames[foundindex] -- number to name
-                            if reference then
-                                local foundindex = lpegmatch(oparser,reference)
-                                if foundindex then
-                                    unicode = cidcodes[foundindex]
-                                    if unicode then
-                                        glyph.unicode = unicode
-                                        ns            = ns + 1
+             -- glyph.name    = overload.name
+                glyph.unicode = overload.unicode
+            else
+                local gu = glyph.unicode -- can already be set (number or table)
+                if not gu or gu == -1 or du >= private or (du >= 0xE000 and du <= 0xF8FF) or du == 0xFFFE or du == 0xFFFF then
+                    local unicode = unicodevector[name] or contextvector[name]
+                    if unicode then
+                        glyph.unicode = unicode
+                        ns            = ns + 1
+                    end
+                    -- cidmap heuristics, beware, there is no guarantee for a match unless
+                    -- the chain resolves
+                    if (not unicode) and usedmap then
+                        local foundindex = lpegmatch(oparser,name)
+                        if foundindex then
+                            unicode = cidcodes[foundindex] -- name to number
+                            if unicode then
+                                glyph.unicode = unicode
+                                ns            = ns + 1
+                            else
+                                local reference = cidnames[foundindex] -- number to name
+                                if reference then
+                                    local foundindex = lpegmatch(oparser,reference)
+                                    if foundindex then
+                                        unicode = cidcodes[foundindex]
+                                        if unicode then
+                                            glyph.unicode = unicode
+                                            ns            = ns + 1
+                                        end
                                     end
-                                end
-                                if not unicode or unicode == "" then
-                                    local foundcodes, multiple = lpegmatch(uparser,reference)
-                                    if foundcodes then
-                                        glyph.unicode = foundcodes
-                                        if multiple then
-                                            nl      = nl + 1
-                                            unicode = true
-                                        else
-                                            ns      = ns + 1
-                                            unicode = foundcodes
+                                    if not unicode or unicode == "" then
+                                        local foundcodes, multiple = lpegmatch(uparser,reference)
+                                        if foundcodes then
+                                            glyph.unicode = foundcodes
+                                            if multiple then
+                                                nl      = nl + 1
+                                                unicode = true
+                                            else
+                                                ns      = ns + 1
+                                                unicode = foundcodes
+                                            end
                                         end
                                     end
                                 end
                             end
                         end
                     end
-                end
-                -- a.whatever or a_b_c.whatever or a_b_c (no numbers) a.b_
-                --
-                -- It is not trivial to find a solution that suits all fonts. We tried several alternatives
-                -- and this one seems to work reasonable also with fonts that use less standardized naming
-                -- schemes. The extra private test is tested by KE and seems to work okay with non-typical
-                -- fonts as well.
-                --
-                if not unicode or unicode == "" then
-                    local split  = lpegmatch(namesplitter,name)
-                    local nsplit = split and #split or 0 -- add if
-                    if nsplit == 0 then
-                        -- skip
-                    elseif nsplit == 1 then
-                        local base = split[1]
-                        local u = unicodes[base] or unicodevector[base] or contextvector[name]
-                        if not u then
+                    -- a.whatever or a_b_c.whatever or a_b_c (no numbers) a.b_
+                    --
+                    -- It is not trivial to find a solution that suits all fonts. We tried several alternatives
+                    -- and this one seems to work reasonable also with fonts that use less standardized naming
+                    -- schemes. The extra private test is tested by KE and seems to work okay with non-typical
+                    -- fonts as well.
+                    --
+                    if not unicode or unicode == "" then
+                        local split  = lpegmatch(namesplitter,name)
+                        local nsplit = split and #split or 0 -- add if
+                        if nsplit == 0 then
                             -- skip
-                        elseif type(u) == "table" then
-                            -- unlikely
-                            if u[1] < private then
+                        elseif nsplit == 1 then
+                            local base = split[1]
+                            local u = unicodes[base] or unicodevector[base] or contextvector[name]
+                            if not u then
+                                -- skip
+                            elseif type(u) == "table" then
+                                -- unlikely
+                                if u[1] < private then
+                                    unicode = u
+                                    glyph.unicode = unicode
+                                end
+                            elseif u < private then
                                 unicode = u
                                 glyph.unicode = unicode
                             end
-                        elseif u < private then
-                            unicode = u
-                            glyph.unicode = unicode
-                        end
-                    else
-                        local t, n = { }, 0
-                        for l=1,nsplit do
-                            local base = split[l]
-                            local u = unicodes[base] or unicodevector[base] or contextvector[name]
-                            if not u then
-                                break
-                            elseif type(u) == "table" then
-                                if u[1] >= private then
-                                    break
-                                end
-                                n = n + 1
-                                t[n] = u[1]
-                            else
-                                if u >= private then
-                                    break
-                                end
-                                n = n + 1
-                                t[n] = u
-                            end
-                        end
-                        if n > 0 then
-                            if n == 1 then
-                                unicode = t[1]
-                            else
-                                unicode = t
-                            end
-                            glyph.unicode = unicode
-                        end
-                    end
-                    nl = nl + 1
-                end
-                -- last resort (we might need to catch private here as well)
-                if not unicode or unicode == "" then
-                    local foundcodes, multiple = lpegmatch(uparser,name)
-                    if foundcodes then
-                        glyph.unicode = foundcodes
-                        if multiple then
-                            nl      = nl + 1
-                            unicode = true
                         else
-                            ns      = ns + 1
-                            unicode = foundcodes
+                            local t, n = { }, 0
+                            for l=1,nsplit do
+                                local base = split[l]
+                                local u = unicodes[base] or unicodevector[base] or contextvector[name]
+                                if not u then
+                                    break
+                                elseif type(u) == "table" then
+                                    if u[1] >= private then
+                                        break
+                                    end
+                                    n = n + 1
+                                    t[n] = u[1]
+                                else
+                                    if u >= private then
+                                        break
+                                    end
+                                    n = n + 1
+                                    t[n] = u
+                                end
+                            end
+                            if n > 0 then
+                                if n == 1 then
+                                    unicode = t[1]
+                                else
+                                    unicode = t
+                                end
+                                glyph.unicode = unicode
+                            end
+                        end
+                        nl = nl + 1
+                    end
+                    -- last resort (we might need to catch private here as well)
+                    if not unicode or unicode == "" then
+                        local foundcodes, multiple = lpegmatch(uparser,name)
+                        if foundcodes then
+                            glyph.unicode = foundcodes
+                            if multiple then
+                                nl      = nl + 1
+                                unicode = true
+                            else
+                                ns      = ns + 1
+                                unicode = foundcodes
+                            end
                         end
                     end
-                end
-                -- check using substitutes and alternates
-                local r = overloads[unicode]
-                if r then
-                    unicode = r.unicode
-                    glyph.unicode = unicode
-                end
-                --
-                if not unicode then
-                    missing[unic] = true
-                    nofmissing    = nofmissing + 1
+                    -- check using substitutes and alternates
+                    local r = overloads[unicode]
+                    if r then
+                        unicode = r.unicode
+                        glyph.unicode = unicode
+                    end
+                    --
+                    if not unicode then
+                        missing[du] = true
+                        nofmissing  = nofmissing + 1
+                    end
                 end
             end
-        else
-            -- no name
         end
     end
     if type(checklookups) == "function" then
         checklookups(data,missing,nofmissing)
     end
+
     -- todo: go lowercase
+
+    local collected = false
+    local unicoded  = 0
+    for unicode, glyph in next, descriptions do
+        if glyph.class == "ligature" and (force_ligatures or not glyph.unicode) then
+            if not collected then
+                collected = fonts.handlers.otf.readers.getcomponents(data)
+                if not collected then
+                    break
+                end
+            end
+            local u = collected[unicode] -- always tables
+            if u then
+                local n = #u
+                for i=1,n do
+                    if u[i] > private then
+                        n = 0
+                        break
+                    end
+                end
+                if n > 0 then
+                    if n > 1 then
+                        glyph.unicode = u
+                    else
+                        glyph.unicode = u[1]
+                    end
+                    unicoded = unicoded + 1
+                end
+            end
+        end
+    end
+    if trace_mapping and unicoded > 0 then
+        report_fonts("%n ligature tounicode mappings deduced from gsub ligature features",unicoded)
+    end
+
     if trace_mapping then
         for unic, glyph in table.sortedhash(descriptions) do
             local name    = glyph.name
