@@ -2,14 +2,14 @@
 -------------------------------------------------------------------------------
 --         FILE:  luaotfload-configuration.lua
 --  DESCRIPTION:  config file reader
--- REQUIREMENTS:  Luaotfload 2.6 or above
+-- REQUIREMENTS:  Luaotfload 2.7 or above
 --       AUTHOR:  Philipp Gesang, <phg@phi-gamma.net>
 --       AUTHOR:  Dohyun Kim <nomosnomos@gmail.com>
 -------------------------------------------------------------------------------
 --
 
 if not modules then modules = { } end modules ["luaotfload-configuration"] = {
-  version   = "2.6",
+  version   = "2.7",
   comment   = "part of Luaotfload",
   author    = "Philipp Gesang, Dohyun Kim",
   copyright = "Luaotfload Development Team",
@@ -89,7 +89,15 @@ local config_paths = {
 }
 
 local valid_formats = tabletohash {
-  "otf", "ttc", "ttf", "dfont", "afm", "pfb", "pfa",
+  "otf", "ttc", "ttf", "afm", "pfb"
+}
+
+local default_anon_sequence = {
+  "tex", "path", "name"
+}
+
+local valid_resolvers = tabletohash {
+  "tex", "path", "name", "file", "my"
 }
 
 local feature_presets = {
@@ -135,7 +143,6 @@ local registered_loaders = {
   reference  = "reference",
   unpackaged = "unpackaged",
   context    = "context",
-  tl2014     = "tl2014",
 }
 
 local pick_fontloader = function (s)
@@ -189,15 +196,17 @@ local permissible_color_callbacks = {
 
 local default_config = {
   db = {
-    formats     = "otf,ttf,ttc,dfont",
-    scan_local  = false,
-    skip_read   = false,
-    strip       = true,
-    update_live = true,
-    compress    = true,
-    max_fonts   = 2^51,
+    formats         = "otf,ttf,ttc",
+    scan_local      = false,
+    skip_read       = false,
+    strip           = true,
+    update_live     = true,
+    compress        = true,
+    max_fonts       = 2^51,
+    use_fontforge   = false,
   },
   run = {
+    anon_sequence  = default_anon_sequence,
     resolver       = "cached",
     definer        = "patch",
     log_level      = 0,
@@ -369,6 +378,20 @@ local set_default_features = function ()
   return true
 end
 
+local set_fontforge = function ()
+  local names = fonts.names
+  if not names or not names.use_fontforge then
+    --- happens normally on the first run
+    logreport ("log", 4, "db", "Database not present.")
+    return true
+  end
+  local use_ff = config.luaotfload.db.use_fontforge
+  if use_ff == true then
+      logreport ("both", 0, "db",
+                 "Fontforge loader was requested but not supported anymore.")
+  end
+  return true
+end
 
 reconf_tasks = {
   { "Set the log level"         , set_loglevel         },
@@ -377,6 +400,7 @@ reconf_tasks = {
   { "Set the font filter"       , set_font_filter      },
   { "Install font name resolver", set_name_resolver    },
   { "Set default features"      , set_default_features },
+  { "Set fontforge"             , set_fontforge        },
 }
 
 -------------------------------------------------------------------------------
@@ -475,8 +499,48 @@ local option_spec = {
       out_t     = number_t, --- TODO int_t from 5.3.x on
       transform = tointeger,
     },
+    use_fontforge = { in_t = boolean_t, },
   },
   run = {
+    anon_sequence = {
+      in_t      = string_t,
+      out_t     = table_t,
+      transform = function (s)
+        if s ~= "default" then
+          local bits = { lpegmatch (commasplitter, s) }
+          if next (bits) then
+            local seq = { }
+            local done = { }
+            for i = 1, #bits do
+              local bit = bits [i]
+              if valid_resolvers [bit] then
+                if not done [bit] then
+                  done [bit] = true
+                  seq [#seq + 1] = bit
+                else
+                  logreport ("both", 0, "conf",
+                             "ignoring duplicate resolver %s at position %d \z
+                              in lookup sequence",
+                             bit, i)
+                end
+              else
+                logreport ("both", 0, "conf",
+                           "lookup sequence contains invalid item %s \z
+                            at position %d.",
+                           bit, i)
+              end
+            end
+            if next (seq) then
+              logreport ("both", 2, "conf",
+                         "overriding anon lookup sequence %s.",
+                         tableconcat (seq, ","))
+              return seq
+            end
+          end
+        end
+        return default_anon_sequence
+      end
+    },
     live = { in_t = boolean_t, }, --- false for the tool, true for TeX run
     resolver = {
       in_t      = string_t,
@@ -614,6 +678,15 @@ local format_keyval = function (var, val)
   end
 end
 
+local format_list = function (var, val)
+  local elts = { }
+  for i = 1, #val do elts [i] = val [i] end
+  if next (elts) then
+    return stringformat (indent .. "%s = %s",
+                         var, tableconcat (elts, ","))
+  end
+end
+
 local format_section = function (title)
   return stringformat ("[%s]", dashed (title))
 end
@@ -643,13 +716,14 @@ local conf_footer = [==[
 
 local formatters = {
   db = {
-    compress    = { false, format_boolean },
-    formats     = { false, format_string  },
-    max_fonts   = { false, format_integer },
-    scan_local  = { false, format_boolean },
-    skip_read   = { false, format_boolean },
-    strip       = { false, format_boolean },
-    update_live = { false, format_boolean },
+    compress       = { false, format_boolean },
+    formats        = { false, format_string  },
+    max_fonts      = { false, format_integer },
+    scan_local     = { false, format_boolean },
+    skip_read      = { false, format_boolean },
+    strip          = { false, format_boolean },
+    update_live    = { false, format_boolean },
+    use_fontforge  = { false, format_boolean },
   },
   default_features = {
     __default = { true, format_keyval },
@@ -667,6 +741,7 @@ local formatters = {
     lookups_file = { false, format_string },
   },
   run = {
+    anon_sequence   = { false, format_list    },
     color_callback  = { false, format_string  },
     definer         = { false, format_string  },
     fontloader      = { false, format_string  },
