@@ -14,14 +14,68 @@ if not luaotfload then error "this module requires Luaotfload" end
 
 local logreport = luaotfload.log and luaotfload.log.report or print
 
+local lua_reader = function (specification)
+  local fullname = specification.filename or ""
+  if fullname == "" then
+    local forced = specification.forced or ""
+    if forced ~= "" then
+      fullname = specification.name .. "." .. forced
+    else
+      fullname = specification.name
+    end
+  end
+  local fullname = resolvers.findfile (fullname) or ""
+  if fullname ~= "" then
+    local loader = loadfile (fullname)
+    loader = loader and loader ()
+    return loader and loader (specification)
+  end
+end
+
+local eval_reader = function (specification)
+  local eval = specification.eval
+  if not eval or type (eval) ~= "function" then return nil end
+  logreport ("both", 0, "loaders",
+             "eval: found tfmdata for “%s”, injecting.",
+             specification.name)
+  return eval ()
+end
+
+local unsupported_reader = function (format)
+  return function (specification)
+    logreport ("both", 4, "loaders",
+               "font format “%s” unsupported; cannot load %s.",
+               format, tostring (specification.name))
+  end
+end
+
+local afm_reader = fonts.readers.afm
+
+local afm_loader = function (specification)
+  specification.forced      = "afm"
+  specification.sub         = false
+  specification.forcedname  = file.addsuffix(specification.name, "afm")
+  inspect(specification)
+  return afm_reader (specification, "afm")
+end
+
+local afm_compat_message = function (specification)
+  logreport ("both", 4, "loaders",
+             "PFB format only supported with matching \z
+              AFM; redirecting (“%s”, “afm”).",
+             tostring (specification.name))
+  return afm_loader (specification)
+end
+
 local install_formats = function ()
   local fonts = fonts
   if not fonts then return false end
 
-  local readers  = fonts.readers
-  local handlers = fonts.handlers
-  local formats  = fonts.formats
-  if not readers or not handlers or not formats then return false end
+  local readers   = fonts.readers
+  local sequence  = readers.sequence
+  local seqset    = table.tohash (sequence)
+  local formats   = fonts.formats
+  if not readers or not formats then return false end
 
   local aux = function (which, reader)
     if   not which  or type (which) ~= "string"
@@ -31,15 +85,40 @@ local install_formats = function ()
     end
     formats  [which] = "type1"
     readers  [which] = reader
-    handlers [which] = { }
+    if not seqset [which] then
+      logreport ("both", 3, "loaders",
+                 "Extending reader sequence for “%s”.", which)
+      sequence [#sequence + 1] = which
+      seqset   [which]         = true
+    end
     return true
   end
 
-  return aux ("pfa", function (spec) return readers.opentype (spec, "pfa", "type1") end)
-     and aux ("pfb", function (spec) return readers.opentype (spec, "pfb", "type1") end)
+  return aux ("evl", eval_reader)
+     and aux ("lua", lua_reader)
+     and aux ("pfa", unsupported_reader "pfa")
+     and aux ("afm", afm_loader)
+     and aux ("pfb", afm_compat_message) --- pfb loader is incomplete
      and aux ("ofm", readers.tfm)
+     and aux ("dfont", unsupported_reader "dfont")
 end
 
+local not_found_msg = function (specification, size, id)
+  logreport ("both", 0, "loaders", "")
+  logreport ("both", 0, "loaders",
+             "--------------------------------------------------------")
+  logreport ("both", 0, "loaders", "")
+  logreport ("both", 0, "loaders", "Font definition failed for:")
+  logreport ("both", 0, "loaders", "")
+  logreport ("both", 0, "loaders", "   > id            : %d", id)
+  logreport ("both", 0, "loaders", "   > specification : %q", specification)
+  if size > 0 then
+    logreport ("both", 0, "loaders", "   > size          : %.2f pt", size / 2^16)
+  end
+  logreport ("both", 0, "loaders", "")
+  logreport ("both", 0, "loaders",
+             "--------------------------------------------------------")
+end
 --[[doc--
 
     \subsection{\CONTEXT override}
@@ -56,6 +135,7 @@ do
 
   local patch = function (specification, size, id)
     local fontdata = read (specification, size, id)
+----if not fontdata then not_found_msg (specification, size, id) end
     if type (fontdata) == "table" and fontdata.shared then
       --- We need to test for the “shared” field here
       --- or else the fontspec capheight callback will
@@ -75,10 +155,8 @@ do
       logreport ("both", 0, "loaders", "   > spec %q", specification)
       logreport ("both", 0, "loaders", "   > at size %.2f pt", size / 2^16)
       local result = definer (specification, size, id)
-      if not result then
-        logreport ("both", 0, "loaders", "   > font definition failed")
-        return
-      elseif type (result) == "number" then
+      if not result then return not_found_msg (specification, size, id) end
+      if type (result) == "number" then
         logreport ("both", 0, "loaders", "   > font definition yielded id %d", result)
         return result
       end
@@ -86,6 +164,7 @@ do
       logreport ("both", 0, "loaders", "   > name %q",     result.name     or "<nil>")
       logreport ("both", 0, "loaders", "   > fontname %q", result.fontname or "<nil>")
       logreport ("both", 0, "loaders", "   > fullname %q", result.fullname or "<nil>")
+      logreport ("both", 0, "loaders", "   > type %s",     result.type     or "<nil>")
       return result
     end
   end
