@@ -1,6 +1,6 @@
-# $Id: TLPSRC.pm 34044 2014-05-15 17:35:07Z karl $
+# $Id: TLPSRC.pm 41619 2016-07-03 23:33:41Z karl $
 # TeXLive::TLPSRC.pm - module for handling tlpsrc files
-# Copyright 2007-2014 Norbert Preining
+# Copyright 2007-2016 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 
@@ -15,7 +15,7 @@ use TeXLive::TLTREE;
 my $_tmp;
 my %autopatterns;  # computed once internally
 
-my $svnrev = '$Revision: 34044 $';
+my $svnrev = '$Revision: 41619 $';
 my $_modulerevision;
 if ($svnrev =~ m/: ([0-9]+) /) {
   $_modulerevision = $1;
@@ -106,7 +106,7 @@ sub from_file {
     # (blank lines are significant in tlpobj, but not tlpsrc)
 
     if ($line =~ /^ /) {
-      die "$srcfile: non-continuation indentation not allowed: `$line'";
+      die "$srcfile:$lineno: non-continuation indentation not allowed: `$line'";
     }
     # remove terminal white space
     $line =~ s/\s+$//;
@@ -262,6 +262,24 @@ sub make_tlpobj {
   $tlp->postactions(@{$self->{'postactions'}}) if (defined($self->{'postactions'}));
   $tlp->depends(@{$self->{'depends'}}) if (defined($self->{'depends'}));
   $tlp->revision(0);
+
+  # convert each fmttrigger to a depend line, if not already present.
+  if (defined($tlp->executes)) { # else no fmttriggers
+    my @deps = (defined($tlp->depends) ? $tlp->depends : ());
+    for my $e ($tlp->executes) {
+      # we only check for AddFormat lines
+      if ($e =~ m/^\s*AddFormat\s+(.*)\s*$/) {
+        my %fmtline = TeXLive::TLUtils::parse_AddFormat_line($1);
+        if (defined($fmtline{"error"})) {
+          tlwarn ("error in parsing $e for return hash: $fmtline{error}\n");
+        } else {
+          TeXLive::TLUtils::push_uniq (\@deps, @{$fmtline{'fmttriggers'}});
+          $tlp->depends(@deps);
+        }
+      }
+    }
+  }
+
   my $filemax;
   my $usedefault;
   my @allpospats;
